<?php

namespace Drupal\mongodb_indexer\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class SettingsController.
 *
 * @package Drupal\mongodb_indexer\Controller
 */
class IndexerController extends ControllerBase {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('mongodb_indexer.engine')
    );
  }


  /**
   * Route for operating system to call using CRON or similar
   *
   * @return JsonResponse
   */
  public function cron() {

    $engine = \Drupal::service('mongodb_indexer.engine');
    $config = \Drupal::config('mongodb_indexer.settings');

    // Set up maximum duration of the lock before it expires
    $lock_duration = $config->get('mongodb_indexer_lock_expire');
    $processed_items = 0;

    $lock = \Drupal::lock();

    $response = new \stdClass();


    // Allow cron the perform only if we have the lock
    if($lock->acquire('mongodb_indexer_cron', $lock_duration)) {
      $started = time();

      $processed_items += $engine->indexQueuedItems();
      $engine->indexQueuedReindexItems();

      // Release the lock
      $lock->release('mongodb_indexer_cron');

      $response->message = "Indexed $processed_items items(s).";
      $response->started = $started;
      $response->ended = time();
      $response->status = 'Done';

    }
    else {
      $response->message = 'Lock in place';
      $response->status = 'Skipped';
    }


    return new JsonResponse($response);
  }

}
